import sys
import subprocess
from subprocess import check_output
from dvc.repo import Repo as DVCRepo
import getpass
import time
from dvc_cc.bcolors import *

SCRIPT_NAME = 'dvc-cc git'
TITLE = 'tools'
DESCRIPTION = 'With this script you can call everything what you can do with git. If you call '+bcolors.OKBLUE+'dvc-cc git branch'+bcolors.ENDC+' it will ' \
              'show you only your working branches and ignore DVC-CC branches. If you call '+bcolors.OKBLUE+'dvc-cc git sync'+bcolors.ENDC+' it will sync ' \
              'all remote branches and create a local branch for this. All other commands are piped to git directly and ' \
              'run '+bcolors.OKBLUE+'dvc checkout'+bcolors.ENDC+' after this to clean up your branches.'


def get_name_of_branch():
    out = check_output(["git", "branch"]).decode("utf8")
    current = next(line for line in out.split("\n") if line.startswith("*"))
    return current.strip("*").strip()


def main():
    argv = sys.argv[1:]
    if '-h' in argv or '--help' in argv or len(argv) == 0:
      print(DESCRIPTION)
      print()
      print('dvc-cc git branch:')
      print('\tShows the branches without the automatic created branches from DVC-CC.')
      print('dvc-cc git sync [-d] [-l]:')
      print('\tCreate local branches for all remote branches.')
      print('\t\t-d: Than it will download all files from the DVC-Server.')
      print('\t\t-l: If this is set, than it will repeat every 20 seconds the script.')
      print('\t\t\tYou can cancel it with CTRL+C.')
      print('dvc-cc git OTHER_GIT_COMMAND:')
      print('\tEvery other git command will be piped directly to git. After it was called it will run '+bcolors.OKBLUE+'dvc checkout'+bcolors.ENDC)
      print('\t\tto cleanup the repository')
    elif len(argv) == 1 and sys.argv[1] == 'branch':
        git_branch = check_output(['git','branch']).decode("utf8").split('\n')
        for line in git_branch:
            if not line.startswith('  rcc_') and not line.startswith('  remotes/origin/rcc_') and not line.startswith('  cc_') and not line.startswith('  remotes/origin/cc_'):
                print(line)
    elif sys.argv[1] == 'sync':
        repo = DVCRepo()
        if (len(argv) > 2 and argv[1] == '-d') or (len(argv) == 3 and argv[2] == '-d'):
            remote_name = repo.config.config['core']['remote']
            remote_settings = repo.config.config['remote "' + remote_name + '"']
            if 'ask_password' in remote_settings and remote_settings['ask_password']:
                remote_settings['password'] = getpass.getpass('Password for ' + remote_settings['url'] + ': ')
                remote_settings['ask_password'] = False

        git_name_of_branch = get_name_of_branch()

        if (len(argv) > 2 and argv[1] == '-l') or (len(argv) == 3 and argv[2] == '-l'):
            loop = True
        else:
            loop = False

        try:
            is_first_iteration = True
            while loop or is_first_iteration:

                if is_first_iteration == False:
                    print('All remote branches were created locally. Wait 5 seconds for the next pull request. To cancel the script press CTRL+C.')
                    time.sleep(5)
                is_first_iteration = False

                _ = check_output(["git", "pull"]).decode("utf8").split("\n")

                all_branches = check_output(["git", "branch", '-a']).decode("utf8").split("\n")
                all_branches_local = [i[2:] for i in all_branches if len(i.split('/')) == 1]
                all_branches_remote = [i.split('/')[-1] for i in all_branches if len(i.split('/')) > 1]


                for b in all_branches_remote:
                    if b not in all_branches_local:
                        print('git checkout '+ b)
                        _ = check_output(['git', 'checkout', b])
                        print('\t\ŧI CHECKOUT THE DATA')
                        repo.checkout()
                        if argv[1] == '-d':
                            print('\t\ŧI PULL THE DATA')
                            repo.pull()
        finally:
            print('git checkout ' + git_name_of_branch)
            _ = check_output(['git', 'checkout', git_name_of_branch])
            repo.checkout()
    else:
        subprocess.call(['git'] + argv)
        subprocess.call(['dvc', 'checkout'])



from collections import OrderedDict
from dvc_cc.version import VERSION
from dvc_cc.cli_modes import cli_modes
from dvc_cc.bcolors import  *

from dvc_cc.hyperopt.new import main as new_main
from dvc_cc.hyperopt.new import DESCRIPTION as NEW_DESCRIPTION

from dvc_cc.hyperopt.var import main as variable_main
from dvc_cc.hyperopt.var import DESCRIPTION as VARIABLE_DESCRIPTION

from dvc_cc.hyperopt.plot_beta_distribution import main as plot_beta_distribution_main
from dvc_cc.hyperopt.plot_beta_distribution import DESCRIPTION as PLOT_BETA_DESCRIPTION

SCRIPT_NAME = 'dvc-cc hyperopt'
TITLE = 'tools'
DESCRIPTION = 'With '+bcolors.OKBLUE+'dvc-cc hyperopt'+bcolors.ENDC+' you can create DVC files with and without hyperparameter that you can than run with ' \
              +bcolors.OKBLUE+'dvc-cc run'+bcolors.ENDC+'.'
MODES = OrderedDict([
    ('new', {'main': new_main, 'description': NEW_DESCRIPTION}),
    #('stage', {'main': stage_main, 'description': STAGE_DESCRIPTION}),
    ('var', {'main': variable_main, 'description': VARIABLE_DESCRIPTION}),
    #('command', {'main': command_main, 'description': COMMAND_DESCRIPTION}),
    ('plot-beta', {'main': plot_beta_distribution_main, 'description': PLOT_BETA_DESCRIPTION})
])


def main():
    cli_modes(SCRIPT_NAME, TITLE, DESCRIPTION, MODES, VERSION)



#!/usr/bin/env python3
from argparse import ArgumentParser
import keyring
import requests
import yaml
from dvc_cc.bcolors import *

DESCRIPTION = 'Cancel jobs that are running or registered on a CC server.'

def read_execution_engine():
    with open('.dvc_cc/cc_config.yml') as f:
        y = yaml.safe_load(f.read())
    return y['execution']['settings']['access']['url']

def main():
    parser = ArgumentParser(description=DESCRIPTION)
    parser.add_argument('-ids','--experiment-ids', help='A list of CC IDs of the experiments. Run "dvc-cc status -id" to show the ID for the experiments.', nargs="+", type=str)
    parser.add_argument('-sids','--subexperiment-ids', help='A list of CC SUB-IDs of the experiments. Run "dvc-cc status -id" to show the SUB-ID for the experiments.', nargs="+", type=str)
    parser.add_argument('--all', help='If this parameter is set, all running or registered jobs get canceled.', default=False, action='store_true')
    parser.add_argument('--last', help='If this parameter is true, than the last started job get canceled.', default=False, action='store_true')
    parser.add_argument('--last_n', help='If you set this parameter it will cancel the last n jobs, that was started.', default=None, type=int)
    args = parser.parse_args()

    pw = keyring.get_password('red', 'agency_password')
    uname = keyring.get_password('red', 'agency_username')
    auth = (uname, pw)

    execution_engine = read_execution_engine()

    if args.last or args.last_n is not None:
        if args.last_n is None:
            args.last_n = 1

        r = requests.get(
            execution_engine+'/experiments',
            auth=auth
        )
        r.raise_for_status()
        data = r.json()

        if args.experiment_ids is None:
            args.experiment_ids = []
        for i in range(args.last_n):
            args.experiment_ids.append(data[i]['_id'])

    if (args.experiment_ids is None or len(args.experiment_ids) == 0) and (args.subexperiment_ids is None or len(args.subexperiment_ids) == 0) and args.all == False:
        print(bcolors.FAIL + 'Error: You need to set one of the following parameters: "-ids", "-sids", "--all", "--last" or "--last_n"' + bcolors.ENDC)
        print()
        print(parser.print_help())
        exit(1)
    
    if args.subexperiment_ids is not None and len(args.subexperiment_ids) > 0:
        sids = args.subexperiment_ids
    else:
        sids = []

    if args.experiment_ids is not None and len(args.experiment_ids) > 0:
        for i in range(len(args.experiment_ids)):
            r = requests.get(
                execution_engine+'/batches?experimentId={}'.format(args.experiment_ids[i]),
                auth=auth
            )
            r.raise_for_status()
            data = r.json()
    
            for d in data:
                if d['state'] != 'succeeded' and d['state'] != 'failed' and d['state'] != 'cancelled':
                    sids.append(d['_id'])

    if args.all:
        r = requests.get(
            execution_engine+'/batches',
            auth=auth
        )
        r.raise_for_status()
        data = r.json()
        for d in data:
            if d['state'] != 'succeeded' and d['state'] != 'failed' and d['state'] != 'cancelled':
                sids.append(d['_id'])
        

    for sid in sids:
        print('DELETE Job: '+execution_engine+'/batches/'+sid)
        r = requests.delete(
            execution_engine+'/batches/'+sid,
            auth=auth
        )
        r.raise_for_status()







import sys
from argparse import ArgumentParser

def cli_modes(script_name, title, description, modes, version=None):
    sys.argv[0] = script_name

    parser = ArgumentParser(description=description)
    if version:
        parser.add_argument(
            '-v', '--version', action='version', version=version
        )
    subparsers = parser.add_subparsers(title=title)

    sub_parser = None
    for key, val in modes.items():
        sub_parser = subparsers.add_parser(key, help=val['description'], add_help=False)

    if len(sys.argv) < 2:
        parser.print_help()
        exit()

    _ = parser.parse_known_args()
    sub_args = sub_parser.parse_known_args()

    mode = modes[sub_args[1][0]]['main']
    sys.argv[0] = '{} {}'.format(script_name, sys.argv[1])
    del sys.argv[1]
    exit(mode())



